(function(){
    'use strict';
    var fs = require('fs'),
        path = require('path'),
        mockRequestGenerator = require('json-schema-test-data-generator');
 
    /**
     * init the module with a swagger spec, flattened paths and outputFolder
     * @param spec
     * @param flatPaths
     * @param outputBase
	 * @param pathParamValuePath
     * @returns {*}
     */
    module.exports = function(spec, flatPaths, outputBase, pathParamValuePath) {

        /**
         * @description generate a mock-object for a given path / operation combination based on it's parameter schema
         * @param apiPath {string} path to generate for
         * @param operation {string} operation to generate for
         * @param successStatusCode {number} The http status-code which is returned for succes
         * @param validationErrorStatusCode {number} The http status-code which is returned for request validation errors
         * @param writeMockToFile {boolean} Should we write the mockfile to the output path
		 * @returns {object} the mock for the path / operation combi.
         *      Contains 2 properties:
         *      - [successStatusCode]: contains the mocks that should lead to a succesful result
         *      - [validationErrorStatusCode]: contains the mocks that should lead to a unsuccesful result, with validation errors
         */
        function generateRequestMock(apiPath, operation, writeMockToFile) {
            var pathMock =  {}, getMocks = {}, codes = [];
			console.log('DENTRO GENERATEREQUESTMOCK <------------------');
			var successStatusCodeArray = [200,201,204];
			var requiredParamArray = [];
			var contaParam = 0 
			var contaPathParam = 0
			var jsonContent;
			var pathParams = false;
			console.log('++++++' + apiPath + ' - ' + operation);
			
			if (pathParamValuePath != undefined && pathParamValuePath != '') {
				try {
					var file_content = fs.readFileSync(pathParamValuePath);
					jsonContent = JSON.parse(file_content);
				} catch (err) {
					console.log('Il file non esiste')
					process.exit()
				}
			}
			
            //pathMock[validationErrorStatusCode] = [];
			//console.log("+++++++++++++++++++++++++++++++++++++++++++++++++++");
			//console.log(JSON.stringify(spec , null, 2));
            if(spec['paths'][apiPath][operation].parameters) {
				contaParam = spec['paths'][apiPath][operation].parameters.length;
				//read first response code
				var firstCode = Object.keys(spec['paths'][apiPath][operation]['responses'])[0]
				//console.log(firstCode)
				//process.exit()
                spec['paths'][apiPath][operation].parameters.forEach(function(param) {
					var objToRead = {}
					if (jsonContent[apiPath] && jsonContent[apiPath][operation] && jsonContent[apiPath][operation][firstCode] && jsonContent[apiPath][operation][firstCode][0]) {
						objToRead = jsonContent[apiPath][operation][firstCode][0]
					}
					if(!jsonContent || !Object.keys(objToRead).includes(param.name)) {
						//console.log('genero' + param.name)
						getMocks[param.name] = (param.schema) ? mockRequestGenerator(param.schema) : mockRequestGenerator(param);
						getMocks[param.name].forEach(function(val) {
							val.in = param.in;
							val.paramName = param.name;
							if (param['x-example'] && val.valid) {
								val.data = param['x-example']
							}
						});
						if (param.required) 
							requiredParamArray.push(param.name);
						else {
							getMocks[param.name].forEach(function (obj) {
								if (obj.data == null)
									obj.valid = true;
							});
						}						
					}
					else {
						if (param.in == 'path') contaPathParam++
						pathParams = true;
						requiredParamArray.push(param.name);
						//c'è un parametro path...allora leggi il file json dei path param passati a mano				
					}
				});
				codes = Object.keys(spec['paths'][apiPath][operation].responses);
				console.log(codes);
				//console.log(requiredParamArray.length + '/' + contaParam);
            }
			

            if(Object.keys(getMocks).length > 0 || pathParams) {
				
				//aggiunge gli apostrofi agli estremi dei valori altrimenti succede che nei test compare:
				//...b: #fgT[9!yZj&oQi... e cerca #fgT[9!yZj&oQi come se fosse una variabile...invece dovrebbe essere
				//...b: '#fgT[9!yZj&oQi'...
				
				Object.keys(getMocks).forEach(function(paramName) {	
					getMocks[paramName].forEach(function(mockObj) {
						mockObj.paramName = paramName;	
						/*
						if(mockObj.in == 'query' && mockObj.data != null && typeof mockObj.data === 'string') {
							mockObj.data = "'" +mockObj.data + "'";
						}
						*/
					});
				});	
				
				console.log(JSON.stringify(getMocks , null, 2));
				/*
				if(pathParams) {
					var file_content = fs.readFileSync(outputBase + '/path_param_skeleton.json');
					jsonContent = JSON.parse(file_content);
					//console.log(JSON.stringify(jsonContent , null, 2));			
				}
				*/
				//happy path - genera i test per 2xx con tutti i parametri validi (anche i non required)
				var validMocks = {};
                Object.keys(getMocks).forEach(function(paramName) {

                    getMocks[paramName].filter(function (fullMock) { return fullMock.valid; }).forEach(function (mock) {
                        if(mock.data != null) {
							if(mock.in == 'body') {
								//se il parametro è un body salva il parametro sotto 'body' e non sotto il suo nome
								validMocks['body'] = mock.data;
							}
							else validMocks[paramName] = mock.data;
							validMocks.description = mock.message;
						}
                    });			
				});
				//cerca il codice di success (200, 201, ...)
				var successStatusCode;
				for(var i = 0; i < codes.length; i++) {
					if(successStatusCodeArray.indexOf(codes[i])) {
						successStatusCode = codes[i];
						break;
					}
				}
				//console.log(successStatusCode);
				pathMock[successStatusCode] = [];
				
				if(jsonContent && jsonContent[apiPath] && jsonContent[apiPath][operation] && jsonContent[apiPath][operation][successStatusCode]) {
					//console.log('+++++++++++++++++++++++++++++++++++++++++++2');
					//console.log(apiPath);
					var pathparamFileContent = jsonContent[apiPath][operation][successStatusCode];
					//console.log(pathparamFileContent);
					pathparamFileContent.forEach(function (obj) {
						var newValidMocks = JSON.parse(JSON.stringify(validMocks));
						Object.keys(obj).forEach(function (key) {
							//console.log(key + ' : ' + obj[key]);
							newValidMocks[key] = obj[key]
						});
						if (newValidMocks.description == undefined)
							newValidMocks.description = '';
						pathMock[successStatusCode].push(newValidMocks);
					});
				}
				else pathMock[successStatusCode].push(validMocks);
				//console.log(JSON.stringify(pathMock , null, 2));
				
				//prepara i dati validi per i soli parametri obbligatori
				//se ci sono parametri non abbligatori
				//console.log(requiredParamArray)
				//console.log(requiredParamArray.length)
				//console.log(contaParam)
				if(requiredParamArray.length != contaParam) {
					//console.log('dentro');
					var validMocks = {};
					//se il parametro è required leggi dal mock valido
					Object.keys(getMocks).forEach(function(paramName) {
						//console.log(paramName)
						if(requiredParamArray.includes(paramName)) {
							//console.log('if')
							getMocks[paramName].filter(function (fullMock) { return fullMock.valid; }).forEach(function (mock) {
								if(mock.in == 'body' ) {
									//se il parametro è un body salva il parametro sotto 'body' e non sotto il suo nome
									validMocks['body'] = mock.data;
								}
								else validMocks[paramName] = mock.data;
								validMocks.description = mock.message;
							});
						}
						//altrimenti assegna null al parametro !required
						else {
							//console.log('else')
							validMocks[paramName] = null;
						}
					});
					if (!('description' in validMocks))
						validMocks.description = 'should work without not required parameters';
					if(jsonContent && jsonContent[apiPath] && jsonContent[apiPath][operation] && jsonContent[apiPath][operation][successStatusCode]) {
						//console.log('+++++++++++++++++++++++++++++++++++++++++++2');
						//console.log(apiPath);
						var pathparamFileContent = jsonContent[apiPath][operation][successStatusCode];
						//console.log(pathparamFileContent);
						pathparamFileContent.forEach(function (obj) {
							var newValidMocks = JSON.parse(JSON.stringify(validMocks));
							Object.keys(obj).forEach(function (key) {
								//console.log(key + ' : ' + obj[key]);
								newValidMocks[key] = obj[key]
							});
							if (newValidMocks.description == undefined)
								newValidMocks.description = '';
							pathMock[successStatusCode].push(newValidMocks);
						});
					}
					else pathMock[successStatusCode].push(validMocks);
					//console.log(JSON.stringify(pathMock , null, 2));
				}
				

				
				var validationErrorStatusCode;
				//bad path
				if(contaParam > 1) {
				//console.log("più parametri crea le combinazioni sbagliate");
					//genera le richieste in modo che n-1 parametri sono validi e sono uno è non valido generando cosi tutte le permutazioni
					Object.keys(getMocks).forEach(function(paramName) {
						var arrayDaPermutare = [];
						var invalidValues = getMocks[paramName].filter(function (fullMock) { return !fullMock.valid;})
						arrayDaPermutare.push(invalidValues);
						Object.keys(getMocks).forEach(function(paramName2) {
								if (paramName != paramName2) {
									arrayDaPermutare.push(getMocks[paramName2].filter(function(fullMock) { return fullMock.valid;}))
								}
						});
						//console.log("===========================");
						//console.log(arrayDaPermutare);
						//console.log("____________________________");
						var permutazioni = cartesian(arrayDaPermutare);
						//console.log("===========================");
						//console.log(permutazioni);
						//console.log("____________________________");
						
						for(var i = 0; i < permutazioni.length; i++) {
							var inValidMocks = {};
							for(var j=0; j < permutazioni[i].length; j++) {
								if (permutazioni[i][j].in == 'body' )
									inValidMocks['body'] = permutazioni[i][j].data;
								else
									inValidMocks[permutazioni[i][j].paramName] = permutazioni[i][j].data;
								if(!permutazioni[i][j].valid) {
									inValidMocks.description = permutazioni[i][j].message + ' of ' + permutazioni[i][j].paramName;
									
									if(permutazioni[i][j].level == 'lexical' || permutazioni[i][j].level == 'syntax') {
										//console.log('-------------------------+ ' + codes)
										//console.log('-------------------------> ' + spec['paths'][apiPath][operation].responses['401']['x-onError'])
										var foundErrorTag = false
										codes.forEach(function (code) {
											//console.log(code)
											if (spec['paths'][apiPath][operation].responses[code]['x-onError']) {
												if (spec['paths'][apiPath][operation].responses[code]['x-onError'].includes(permutazioni[i][j].paramName)) {
													//console.log('!!!!!!!!!!!!!!!!!!!!!!!!!x-onError trovato!!!!!!!!!!!!!!!!!!!!!!! ' + code + ' ' + permutazioni[i][j].paramName)
													validationErrorStatusCode = code
													foundErrorTag = true
												}
											}
										})
										
										if (!foundErrorTag) {
											validationErrorStatusCode = 'default';
										}
									}
								}
							}
							if(pathMock[validationErrorStatusCode] == null)
								pathMock[validationErrorStatusCode] = [];
							if(jsonContent && jsonContent[apiPath] && jsonContent[apiPath][operation] && jsonContent[apiPath][operation][validationErrorStatusCode]) {
								//qui aggiungi i path
								//console.log(apiPath)
								//console.log(operation)
								//console.log(validationErrorStatusCode)
								var pathparamFileContent = jsonContent[apiPath][operation][validationErrorStatusCode];							
								pathparamFileContent.forEach(function (obj) {
									var newInValidMocks = JSON.parse(JSON.stringify(inValidMocks));
									Object.keys(obj).forEach(function (key) {
										//console.log(key + ' : ' + obj[key]);
										newInValidMocks[key] = obj[key]
									});
									pathMock[validationErrorStatusCode].push(newInValidMocks);
								});
							}
							else 
								pathMock[validationErrorStatusCode].push(inValidMocks);
							
						}
					});
					//console.log("-------------------------")
					//console.log(JSON.stringify(pathMock[validationErrorStatusCode] , null, 2));
					//console.log("-------------------------")
					//mancano i 4xx presi dal file e permutati con gli invalid mocks
					if(jsonContent && jsonContent[apiPath] && jsonContent[apiPath][operation]) {
						//console.log('+++++++++++++++++++++++++++++++++++++++++++2');
						//console.log(apiPath);
						var pathparamFileContent = jsonContent[apiPath][operation];
						Object.keys(pathparamFileContent).forEach(function (key) {
							//console.log(key);
							if (key == 404 && contaPathParam > 0) {
								//console.log('dentro');
								if (pathMock[key] == null)
										pathMock[key] = [];
								var codeObj = pathparamFileContent[key];
								var arrayDaPermutare = [];
								arrayDaPermutare.push(codeObj);
								Object.keys(getMocks).forEach(function(paramName2) {
									arrayDaPermutare.push(getMocks[paramName2].filter(function(fullMock) { return fullMock.valid;}))
								});
								//console.log("2===========================");
								//console.log(arrayDaPermutare);
								//console.log("2____________________________");
								var permutazioni = cartesian(arrayDaPermutare);
								//console.log("2===========================");
								//console.log(permutazioni);
								//console.log("2____________________________");
								
								permutazioni.forEach(function(arrayElem) {
								var inValidMocks2 = {};
									arrayElem.forEach(function(parametro) {										
										//console.log('su invalid mocks');
										//console.log(parametro);
										if(parametro.data != undefined || parametro.data === null) {
											//console.log('su parametro.data' + parametro.data);
											inValidMocks2[parametro.paramName] = parametro.data;
										}
										else {
											//console.log('else parametro path');
											Object.keys(parametro).forEach(function(chiave) {
												inValidMocks2[chiave] = parametro[chiave];
											});
										}
										inValidMocks2.description = '';
									});
									pathMock[key].push(inValidMocks2);
								});
							
							}
						});
					}
				}
				//se c'è un solo parametro...prova a uno a uno i parametri not valid
				else {
					//console.log("c'è un solo parametro...prova a uno a uno i parametri not valid");
					Object.keys(getMocks).forEach(function(paramName) {

						getMocks[paramName].filter(function (fullMock) { return !fullMock.valid; }).forEach(function (mock) {
							var inValidMocks = {};
							if(mock.in == 'body') {
								//se il parametro è un body salva il parametro sotto 'body' e non sotto il suo nome
								inValidMocks['body'] = mock.data;
							}
							else 
								inValidMocks[paramName] = mock.data;
							inValidMocks.description = mock.message;
							
							if(mock.level == 'lexical' || mock.level == 'syntax') {
								//console.log('-------------------------+ ' + codes)
								//console.log('-------------------------> ' + spec['paths'][apiPath][operation].responses['401']['x-onError'])
								var foundErrorTag = false
								codes.forEach(function (code) {
									//console.log(code)
									if (spec['paths'][apiPath][operation].responses[code]['x-onError']) {
										if (spec['paths'][apiPath][operation].responses[code]['x-onError'].includes(mock.paramName)) {
											//console.log('!!!!!!!!!!!!!!!!!!!!!!!!!x-onError trovato!!!!!!!!!!!!!!!!!!!!!!! ' + code + ' ' + mock.paramName)
											validationErrorStatusCode = code
											foundErrorTag = true
										}
									}
								})
								
								if (!foundErrorTag) {
									validationErrorStatusCode = 'default';
								}
							}														
						
							if(pathMock[validationErrorStatusCode] == null)
								pathMock[validationErrorStatusCode] = [];
							pathMock[validationErrorStatusCode].push(inValidMocks);
							
						});
				
					});
					
					if(jsonContent && jsonContent[apiPath] && jsonContent[apiPath][operation]) {
						//console.log('+++++++++++++++++++++++++++++++++++++++++++2');
						//console.log(apiPath);
						var pathparamFileContent = jsonContent[apiPath][operation];
						Object.keys(pathparamFileContent).forEach(function (key) {
							//console.log(key);
							if (key == 404 && contaPathParam > 0) {
								//console.log('dentro');
								if (pathMock[key] == null)
										pathMock[key] = [];
								var codeObj = pathparamFileContent[key];
								codeObj.forEach(function (value) {
									value.description = '';
									pathMock[key].push(value);
								});
							}
						});
					}
				}
			
                if (writeMockToFile) {
                    fs.writeFileSync(path.join(process.cwd(), outputBase, operation, apiPath) + '/request-array.json', JSON.stringify(getMocks), {}, function (err) {
                        if (err) { return console.log(err); }
                        console.log("The request mock for " + apiPath + operation +" was saved");
                    });
                }
				//console.log("______________________");
				//var permutazioni = cartesian([['b1', 'b2', 'b3']]);
				//console.log(permutazioni);
				//console.log("______________________");
                //console.log('FINE GENERATEREQUESTMOCK <------------------');
				return pathMock;
            }        
		}
		
        return {
            generateRequestMock: generateRequestMock
        };
    } ;
}());

function cartesian(arraysToCombine) {
    var divisors = [];
    for (var i = arraysToCombine.length - 1; i >= 0; i--) {
       divisors[i] = divisors[i + 1] ? divisors[i + 1] * arraysToCombine[i + 1].length : 1;
    }
    
    function getPermutation(n, arraysToCombine) {
       var result = [], 
           curArray;    
       for (var i = 0; i < arraysToCombine.length; i++) {
          curArray = arraysToCombine[i];
          result.push(curArray[Math.floor(n / divisors[i]) % curArray.length]);
       }    
       return result;
    }
    
    var numPerms = arraysToCombine[0].length;
    for(var i = 1; i < arraysToCombine.length; i++) {
        numPerms *= arraysToCombine[i].length;
    }
    
    var combinations = [];
    for(var i = 0; i < numPerms; i++) {
        combinations.push(getPermutation(i, arraysToCombine));
    }
    return combinations;
}

