(function() {
    'use strict';
    var fs = require('fs'),
        mkdirp = require('mkdirp'),
        stt = require('swagger-test-templates'),
        path = require('path'),
        deref = require('json-schema-deref-sync'),
        ZSchema = require('z-schema'),
        zSchema = new ZSchema(),
        swaggerSchema = require('swagger-schema-official/schema'),
        outputBase, spec, flatPaths, responseMockGenerator, requestMockGenerator, pathParamValuePath;
 
    /**
     * @description Generates tests and mocks for the given openApi spec.
     * @param writeMocks {boolean} Whether the module should write the generated requestMocks to file seperatly
     * @param templatesPath {String} Folder with tests-tempates. Needed as long as swagger-test-templates does not support options
	 * @todo remove when stt support request options
     */
    function generate(writeMocks, templatesPath) {
        var requestMocks = {},
            basePath = spec.basePath || '';

        flatPaths.forEach(function(apiPath) {
			if(!requestMocks[apiPath.path]) {
				requestMocks[apiPath.path] = {};
			}
            requestMocks[apiPath.path][apiPath.operation] = requestMockGenerator(apiPath.path, apiPath.operation, writeMocks);
            //responseMockGenerator.generateResponseMock(apiPath.path, apiPath.operation, 200);
        });
		var codes = [];
		for(var path in requestMocks) {
			//console.log(path);
			for(var method in requestMocks[path]) {
				//console.log("  " + method);
				for(var codice in requestMocks[path][method]) {
					//console.log("    " + codice);
					if (codes.indexOf(codice) == -1 )
						codes.push(codice);
				}
			}
		}
		//console.log(codes);
        generateTests(requestMocks, templatesPath, codes);
    }

    /**
     * @description Generates tests for the given openApi spec.
     * @param mocks {object} Mock data ordered by endpoint / operation / responseCode
     * @param templatesPath {String} Folder with tests-tempates. Needed as long as swagger-test-templates does not support options
     */
    function generateTests(mocks, templatesPath, codes) {
		console.log();
		console.log('----------------- requestData -----------------');
		console.log(JSON.stringify(mocks , null, 2));		
        var testConfig  = {
            assertionFormat: 'expect', testModule: 'request', pathName: [], requestData: mocks, maxLen: -1, templatesPath: templatesPath//, statusCodes: [200,400]
        };
		//console.log("fafsdasfadfsadsfdasdfasafdsdfasadfsdfsdfafsdsafdsafdsdaffsadsafd")
        stt.testGen(spec, testConfig).forEach(function(file) {
            //we want spec in the name, not test
            fs.writeFileSync(path.join(outputBase, file.name.replace('test', 'spec')), file.test);
        });
    }

    /**
     * @description Smash nested path / operation info from swaggerJson for easier handling
     * @param  apiPaths {object} the endpoint for the api
     * @param base {string} baseDirectory if filled with a path we will create a directory structure to store mockdata
     * @return {array} paths / operations combinations
     */
    function flattenAndCreateDirectories(apiPaths, base) {
        var responsePaths = [];
        for (var localPath in apiPaths) {
            for ( var operation in apiPaths[localPath]) {
                responsePaths.push({ path: localPath, operation: operation });
                if (base) {
                    mkdirp.sync(path.join(process.cwd(), outputBase, operation, localPath));
                }
            }
        }
        return responsePaths;
    }

    /**
     * @constructor
     * @param openApiSpec {object} A valid openApi Specification json describing your api
     * @param outputPath {string} The base where to write the files to
	 * @param pathParamValueDir {string} The path to the file that contain value for parameters for which generator must not be used
     * @returns {{generate: generate}}
     */
    module.exports = function(openApiSpec, outputPath, pathParamValueDir) {
        if (!openApiSpec) throw new Error('please provide a openAPI json to start');
        if (!zSchema.validate(openApiSpec, swaggerSchema)) throw new Error('openAPI spec not valid');
        if (!outputPath) { throw new Error('please supply an output path'); }

        outputBase = outputPath;
		pathParamValuePath = pathParamValueDir
        spec = deref(openApiSpec);
        flatPaths = flattenAndCreateDirectories(spec.paths, outputBase);

        responseMockGenerator = require('./generateResponseMocks')(spec, flatPaths, outputBase);
        requestMockGenerator = require('./generateRequestMocks')(spec, flatPaths, outputBase, pathParamValuePath).generateRequestMock;

        return {
            generate: generate,
            generateNgApiMockdata: responseMockGenerator.generateNgApiMockData
        }
    };
}());